/*******************************************************************************
 * (c) Copyright 2016 Microsemi Corporation.  All rights reserved.
 *
 */

#include "hw_platform.h"
#include "cortex_nvic.h"
#include "drivers/CoreGPIO/core_gpio.h"
#include "drivers/CoreUARTapb/core_uart_apb.h"
#include "drivers/CoreI2C/core_i2c.h"
#include "drivers/CoreTimer/core_timer.h"
#include "hal.h"
#include "string.h"
#include "stdio.h"
#include "inttypes.h"

#include "./CMSIS/cortexm1_cfg.h"
#include "./CMSIS/system_cortexm1_cfg.h"

#include "mac_uip_functions.h"

#include "uip.h"
#include "uip_arp.h"
#include "uip-split.h"

/* set this define to '1', if you use Everest DEV Board revision A or B,
 * otherwise Everest DEV Board PROTO will be assumed
 */
#define EVEREST_DEV_BOARD_REV_A_OR_B	1

#define DIVISOR 64

#define UART_BASE_ADDRESS 			0x00040000UL
#define GPIO_BASE_ADDRESS    	    0x00041000UL
#define I2C_BASE_ADDRESS    	    0x00042000UL
#define MAC10GB_BASE_ADDRESS    	0x00043000UL
#define MAC10GBFIFO_BASE_ADDRESS   	0x00044000UL
#define TIMER0_BASE_ADDRESS    	    0x00045000UL
#define TIMER1_BASE_ADDRESS    	    0x00046000UL

#define MASTER_SER_ADDRESS    	    0x21
#define BAUD_VALUE                  115200

#if EVEREST_DEV_BOARD_REV_A_OR_B
gpio_instance_t g_gpio_sfp;
#define SFP_CTRL_ADDRESS			0x00049000UL
#define SFP_CTRL_AC0				(GPIO_3)
#define SFP_CTRL_AC1				(GPIO_4)
#define SFP_CTRL_TX_DIS				(GPIO_5)
#endif

gpio_instance_t g_gpio;
UART_instance_t g_uart;
i2c_instance_t g_i2c;
timer_instance_t g_timer0;
timer_instance_t g_timer1;


// global milliseconf timer
uint32_t g_ui32LocalTimer;
volatile uint8_t uip_period_int = 0;


// Cortex-M1 interrupt handler for IRQ0
__attribute__((__interrupt__)) void IRQ0_IRQHandler ( void )
{
	I2C_isr( &g_i2c );
	NVIC_clear_interrupt( NVIC_IRQ_0 );
}

// Cortex-M1 interrupt handler for IRQ1
__attribute__((__interrupt__)) void IRQ1_IRQHandler ( void )
{
	TMR_clear_int (&g_timer0);
	NVIC_clear_interrupt( NVIC_IRQ_1 );
	g_ui32LocalTimer++;
	uip_period_int = 1;							// UIP Periodic Timer fires
	/* Reload Timer */
	TMR_reload(&g_timer0, 50000);
}

/******************************************************************************
 * UIP Declarations
 *****************************************************************************/

void setup_uip_timer()
{
	g_ui32LocalTimer = 0;

	// set timer to millisecond timeout
	TMR_init (&g_timer0, TIMER0_BASE_ADDRESS, TMR_ONE_SHOT_MODE,
			  PRESCALER_DIV_2, 50000);

	// enable interrupt
	NVIC_enable_interrupt(NVIC_IRQ_1);
	TMR_enable_int (&g_timer0);
	TMR_start (&g_timer0);
}


/*-------------------------------------------------------------------------*//**
 * main() function.
 */
int main()
{
    /*
	* CMSIS function- sets firmware system clock to Libero definition
	*/
   volatile uint32_t i, j = 0;
   volatile uint32_t temp = {1};
   volatile uint32_t tx_stats, rx_stats, stats_avail = 0;
   volatile uint32_t gpio_inputs;
   uint8_t  pca9538a_slave_addr = 0x70;
   uint8_t  i2c_tx_buffer[2];
   i2c_status_t status;

   // Initialize GPIO
   GPIO_init ( &g_gpio, GPIO_BASE_ADDRESS, GPIO_APB_32_BITS_BUS);
   GPIO_config ( &g_gpio, GPIO_0, GPIO_INPUT_MODE);
   GPIO_config ( &g_gpio, GPIO_1, GPIO_INPUT_MODE);
   GPIO_config ( &g_gpio, GPIO_2, GPIO_INPUT_MODE);
   GPIO_config ( &g_gpio, GPIO_3, GPIO_INPUT_MODE);
   GPIO_config ( &g_gpio, GPIO_4, GPIO_INPUT_MODE);
   GPIO_config ( &g_gpio, GPIO_5, GPIO_INPUT_MODE);
   GPIO_config ( &g_gpio, GPIO_6, GPIO_INPUT_MODE);
   GPIO_config ( &g_gpio, GPIO_7, GPIO_INPUT_MODE);
   GPIO_config ( &g_gpio, GPIO_8, GPIO_INPUT_MODE);
   GPIO_config ( &g_gpio, GPIO_9, GPIO_INPUT_MODE);
   GPIO_config ( &g_gpio, GPIO_10, GPIO_INPUT_MODE);
   GPIO_config ( &g_gpio, GPIO_11, GPIO_INPUT_MODE);
   GPIO_config ( &g_gpio, GPIO_12, GPIO_INPUT_MODE);
   GPIO_config ( &g_gpio, GPIO_13, GPIO_INPUT_MODE);
   // Read GPIO Value
   gpio_inputs = GPIO_get_inputs(&g_gpio);


   SystemCoreClockUpdate();
   UART_init( &g_uart, UART_BASE_ADDRESS, SYS_M1_CLK_FREQ/((16 * BAUD_VALUE)-1),
		      (DATA_8_BITS | NO_PARITY));

   // Initialize I2C
   I2C_init( &g_i2c, I2C_BASE_ADDRESS, MASTER_SER_ADDRESS, I2C_PCLK_DIV_256);
   NVIC_enable_interrupt(NVIC_IRQ_0);
   /* SFP CTRL */
   // set TX_DISABLE Port to '0' (SFP-Port)
   // set RS0 Port to '1' (SFP-Port)
   // set RS1 Port to '1' (SFP-Port)
#if EVEREST_DEV_BOARD_REV_A_OR_B
	GPIO_init(&g_gpio_sfp, SFP_CTRL_ADDRESS, GPIO_APB_32_BITS_BUS);
	GPIO_set_output(&g_gpio_sfp, SFP_CTRL_AC0, 1);
	GPIO_set_output(&g_gpio_sfp, SFP_CTRL_TX_DIS, 0);
#else
   i2c_tx_buffer[0] = 0x03;
   i2c_tx_buffer[1] = 0xF8;
   I2C_write( &g_i2c, pca9538a_slave_addr, i2c_tx_buffer, 2, I2C_RELEASE_BUS );
   status = I2C_wait_complete( &g_i2c, I2C_NO_TIMEOUT );

   i2c_tx_buffer[0] = 0x01;
   i2c_tx_buffer[1] = 0x03;
   I2C_write( &g_i2c, pca9538a_slave_addr, i2c_tx_buffer, 2, I2C_RELEASE_BUS );
   status = I2C_wait_complete( &g_i2c, I2C_NO_TIMEOUT );
#endif

   // Read GPIO Value
   gpio_inputs = GPIO_get_inputs(&g_gpio);

   // deactivate reset from 10G Core
   GPIO_set_output(&g_gpio, GPIO_31, 1);
   GPIO_set_output(&g_gpio, GPIO_30, 1);
   // Wait for Reset done
   while ((GPIO_get_inputs(&g_gpio) & 0x1FF) != 0x13E );
   // Configure 10GMAC
   configure_10GMAC();
   init_eth_regs();
   //setup_uip_timer();

   while(1)
   {
	   gpio_inputs = GPIO_get_inputs(&g_gpio);
	   // check received packets
	   stats_avail = HW_get_32bit_reg( MAC10GBFIFO_BASE_ADDRESS + 0x24 );
	   if ((stats_avail & 0x1) == 0x0)
	   {
		   // read out status of received packet
		   rx_stats = HW_get_32bit_reg( MAC10GBFIFO_BASE_ADDRESS + 0x20 );
		   // receive packet
	  	   uip_len =  MAC_rx_packet();

	  	   if(BUF->type == htons(UIP_ETHTYPE_IP))
	  	   {
	  	  			uip_arp_ipin();
	  	  			uip_input();
	  	  			if (uip_len > 0)
	  	  			{
	  	  				uip_arp_out();
	  	  			    check_tx_length(&uip_len);
	  	  				MAC_tx_packet(uip_len);
	  	  			}
	  	    }
	  		else if (BUF->type == htons(UIP_ETHTYPE_ARP))
	  	  	{
	  	  			uip_arp_arpin();
	  	  			if (uip_len > 0)
	  	  			{
	  	  			    check_tx_length(&uip_len);
	  	  				MAC_tx_packet(uip_len);
	  	  			}
	  	    }

	  	  	if (uip_period_int > 0)
	  	  	{
	  	  		for (i = 0; i < UIP_CONNS; i++)
	  	  		{
	  	  			uip_poll_conn(i);
	  	  			// uip_periodic(i);
	  	  			if(uip_len > 0)
	  	  			{
	  	  				uip_split_output();
	  	  			}
	  	  		}
	  	  		uip_arp_timer();
	  	  		uip_period_int = 0;
	  		}
	   }
   }

   return 0;
}
